part of tecfy_core_client_package;

abstract class Property {

  String dbName;
  ValueTypes valueType;
  PropertyTypes propertyType;

  bool fullWidth= false;
  int width;

  bool readOnly = false;

  String nameAr;
  String nameEn;
  String get name => LocalConfigService.isArabic ? nameAr : nameEn;

  bool isEdit= false;
  bool isGrid= false;

  bool isRequired= false;

  String validationFormula;

  String editTabName;

  Language language;
  String defaultString;

  bool isSearchable =false;
  bool isDisplayField  = false;

  Widget getDisplayWidget(Map data);

  Property({this.propertyType,this.dbName,this.nameAr,this.nameEn,this.defaultString,this.valueType,this.width,this.fullWidth});
  Property.fromMap(Map row) {
    dbName = lowerFirst(row['dbName']) ?? '';

    valueType = ValueTypes.values[row['valueType']] ?? ValueTypes.Text;
    propertyType = PropertyTypes.values[row['propertyType']] ?? PropertyTypes.Text;
    fullWidth = row['fullWidth'] ?? false;
    width = row['width'] ?? 1;

    readOnly = row['readOnly'] ?? false;

    nameAr = row['nameAr'] ?? dbName;
    nameEn = row['nameEn'] ?? dbName;
    isEdit = row['isEdit'] ?? false;
    isGrid = row['isGrid'] ?? false;

    isRequired = row['isRequired'] ?? false;

    validationFormula = row['validationFormula'];

    editTabName = row['editTabName'];

    language = Language.values[row['language'] ?? 0];
    if (language == null) language = Language.notSelected;

    defaultString = row['default'] ?? '';
    isSearchable = row['isSearchable'] ?? false;
    isDisplayField = row['isDisplayField'] ?? false;
  }

  static Property getProperty(Map row){

    var propertyType = PropertyTypes.values[row['propertyType']];
    switch(propertyType){
      case PropertyTypes.Boolean:
      return PropertyBoolean.fromMap(row);
      case PropertyTypes.Date:
      return PropertyDate.fromMap(row);
      case PropertyTypes.DateTime:
      //return PropertyDate.fromMap(row);
      case PropertyTypes.Dropdown:
      return PropertyDropdown.fromMap(row);
      case PropertyTypes.File:
      return PropertyFile.fromMap(row);
      case PropertyTypes.ManyToMany:
      return PropertyManyToMany.fromMap(row);
      case PropertyTypes.MultiDropdown:
      //return PropertyBoolean.fromMap(row);
      case PropertyTypes.Number:
      return PropertyNumber.fromMap(row);
      case PropertyTypes.OneToMany:
      return PropertyOneToMany.fromMap(row);
      case PropertyTypes.OtherEntityMany:
      return PropertyOtherEntityMany.fromMap(row);
      case PropertyTypes.Text:
      return PropertyText.fromMap(row);
      case PropertyTypes.Time:
      // return PropertyTime.fromMap(row);
      default:
      return null;
    }
  }

  lowerFirst(String s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  upperFirst(String s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toUpperCase()}${s.substring(1)}';
  }

  bool get isImage =>
      this.propertyType == PropertyTypes.File &&
      (this as PropertyFile).isImage == true;

  OperationResult isValid(dynamic data) {
    if ((isRequired && data != null) || (!isRequired))
      return OperationResult(success: true);
    else
      return OperationResult(success: false, message: "$name : Value Required");
  }

  Future<bool> getDefaultValue(Map<String,dynamic> data);
  
  String get filterValueField {
    if(propertyType == PropertyTypes.Dropdown){
      return (this as PropertyDropdown).filterValueField;
    }else if(propertyType == PropertyTypes.ManyToMany){
      return (this as PropertyManyToMany).filterValueField;
    }return "";
  }
}
