part of tecfy_core_client_package;

class EntityService {
  static List<EntityCategory> categories;
  //static List<Entity> entities;
  static Map<String, Entity> entities;
  static Map<String, List<dynamic>> dataCache = {};
  static int pageSize = 15;

  static initEntities() async {

    try {
      var result = await ApiService.httpGetDynamic('entity/GetEntities');
      if (result.success) {
        print('-----------  GetEntities ------------ ${result.data.length}');
        //print(result.data);
        List<Entity> entList=[];
        EntityService.entities = {};
        for (var cat in result.data) {
          var ent = Entity.fromMap(cat);
          entList.add(ent);
          EntityService.entities[ent.dbName] = ent;
        }
        print(EntityService.entities.length);
        for(var c in EntityService.categories){
          c.entities = entList.where((e)=> c.entities.any((ee)=>ee.dbName == e.dbName)).toList();
        }
        print(EntityService.categories.length);
      }
    } catch (err) {
      print('XXXXXXXXXXXXXXX GetEntities $err');
      throw "init entities Error: $err";
    }
  }

  Future<OperationResult<List<dynamic>>> getData(String entity,
      {bool forceFromServer = false, List<EntityFilter> filter, int pageIndex =0, bool activeOnly = false, bool allRecords = false}) async {
    try{
      //forceFromServer =true;
    if (!forceFromServer && (filter == null || filter.length == 0) && !activeOnly && (filter == null || filter.length ==0 )) {
      //return OperationResult<List<dynamic>>(success: true, data: dataCache[entity]);
      var dbResult = await DatabaseService.readCollection(entity);
      if (dbResult.success && dbResult.data != null) {
        return OperationResult<List<dynamic>>(
            success: true, data: dbResult.data);
      }
    }
    var pagingQuery ='?skip=${pageIndex * pageSize }&count=${allRecords ? 10000 : pageSize}&activeOnly=$activeOnly';
    var result = await ApiService.httpGetDynamic('entity/get/$entity' +
        ((filter?.length ?? 0) > 0
            ? '$pagingQuery&filter=${jsonEncode(filter.map<Map>((f) => f.toJson()).toList())}'
            : pagingQuery));
    if (!result.success) {
      print(result.success);
      print(result.message);
      return OperationResult<List<dynamic>>(
          success: false, message: result.message);
    }
    // if(cache) dataCache[entity] =result.data;
    if(pageIndex ==0) await DatabaseService.writeCollection(entity, result.data);
    return OperationResult<List<dynamic>>(success: true, data: result.data);
    }catch(err){
      return OperationResult<List<dynamic>>(success:false, message: err.toString());
    }
  }

  Future<dynamic> getOne(String entity, dynamic id,{bool forView=false}) async {
    var result = await ApiService.httpGetDynamic('entity/GetById/$entity/$id?view=$forView');
    if (!result.success) {
      print(result.success);
      print(result.message);
      return null;
    }
    return result.data;
  }

  Future<OperationResult> delete(String entity, dynamic id) async {
    var result = await ApiService.httpDeleteDynamic('entity/$entity/$id');
    if (!result.success) {
      print(result.success);
      print(result.message);
      return result;
    }
    return result;
  }

  static Entity getEntity(String name) => entities[lowerFirst(name)];

  static OperationResult<bool> validateEntity(Entity entity, dynamic data) {
    print('Validate Entity ${entity.dbName}');
    for (var prop in entity.properties) {
      
      if (prop.propertyType == PropertyTypes.OtherEntityMany) {
        
        var subList = data[prop.dbName];
        if (prop.isRequired && (subList == null || subList.length == 0))
          return OperationResult(
              success: false,
              message: 'Add Atleast one record: ${prop.nameEn}');
        else if (subList != null)
          for (var sub in subList) {
            var result =
                validateEntity(entities[(prop as PropertyOtherEntityMany).entityName], sub);
            if (!result.success) return result;
          }
      } else {
        if (prop.isRequired && data[prop.dbName] == null) {
          return OperationResult(
              success: false, message: 'Field is required: ${prop.nameEn}');
        }

        var result = prop.isValid(data[prop.dbName]);
        if(!result.success)return result;
      }
    }
    return OperationResult(success: true);
  }

  static Future<OperationResult> addEntity(Entity entity, data) async {
    return await ApiService.httpPostEx('entity/${entity.dbName}',
        json.encode(data, toEncodable: customEncode).replaceAll('\"', '\''));
  }

  static Future<OperationResult> saveEntity(Entity entity, data) async {
    return await ApiService.httpPutEx('entity/${entity.dbName}',
        json.encode(data, toEncodable: customEncode).replaceAll('\"', '\''));
  }

  static dynamic customEncode(dynamic item) {
    if (item is DateTime) {
      return DateFormat("yyyy-MM-ddTHH:mm:ss").format(item);
    }
    return item;
  }

  static lowerFirst(String s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }
}
